"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteInstitutionsByInstitutionId = exports.patchInstitutionsByInstitutionId = exports.getInstitutionsByInstitutionId = exports.postInstitutions = exports.getInstitutions = void 0;
const service_common_1 = require("@crosslab/service-common");
const data_source_1 = require("../data_source");
const format_1 = require("../methods/format");
const write_1 = require("../methods/write");
const model_1 = require("../model");
/**
 * This function implements the functionality for handling GET requests on /institutions endpoint.
 * @param _user The user submitting the request.
 */
const getInstitutions = async (_user) => {
    const institutionRepository = data_source_1.AppDataSource.getRepository(model_1.InstitutionModel);
    const institutions = await institutionRepository.find();
    return {
        status: 200,
        body: institutions.map(format_1.formatInstitution),
    };
};
exports.getInstitutions = getInstitutions;
/**
 * This function implements the functionality for handling POST requests on /institutions endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postInstitutions = async (body, _user) => {
    const institutionRepository = data_source_1.AppDataSource.getRepository(model_1.InstitutionModel);
    const institution = institutionRepository.create();
    (0, write_1.writeInstitution)(institution, body);
    await institutionRepository.save(institution);
    return {
        status: 201,
        body: (0, format_1.formatInstitution)(institution),
    };
};
exports.postInstitutions = postInstitutions;
/**
 * This function implements the functionality for handling GET requests on /institutions/{institution_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const getInstitutionsByInstitutionId = async (parameters, _user) => {
    const institutionRepository = data_source_1.AppDataSource.getRepository(model_1.InstitutionModel);
    const institution = await institutionRepository.findOneBy({
        uuid: parameters.institution_id,
    });
    if (!institution) {
        throw new service_common_1.MissingEntityError(`Could not find institution ${parameters.institution_id}`, 404);
    }
    return {
        status: 200,
        body: (0, format_1.formatInstitution)(institution),
    };
};
exports.getInstitutionsByInstitutionId = getInstitutionsByInstitutionId;
/**
 * This function implements the functionality for handling PATCH requests on /institutions/{institution_id} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const patchInstitutionsByInstitutionId = async (parameters, body, _user) => {
    const institutionRepository = data_source_1.AppDataSource.getRepository(model_1.InstitutionModel);
    const institution = await institutionRepository.findOneBy({
        uuid: parameters.institution_id,
    });
    if (!institution) {
        throw new service_common_1.MissingEntityError(`Could not find institution ${parameters.institution_id}`, 404);
    }
    if (body)
        (0, write_1.writeInstitution)(institution, body);
    await institutionRepository.save(institution);
    return {
        status: 200,
        body: (0, format_1.formatInstitution)(institution),
    };
};
exports.patchInstitutionsByInstitutionId = patchInstitutionsByInstitutionId;
/**
 * This function implements the functionality for handling DELETE requests on /institutions/{institution_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const deleteInstitutionsByInstitutionId = async (parameters, _user) => {
    const institutionRepository = data_source_1.AppDataSource.getRepository(model_1.InstitutionModel);
    const result = await institutionRepository.softDelete({
        uuid: parameters.institution_id,
    });
    if (!result.affected) {
        throw new service_common_1.MissingEntityError(`Could not find institution ${parameters.institution_id}`, 404);
    }
    if (result.affected > 1) {
        throw new service_common_1.InconsistentDatabaseError(`More than one institution with id ${parameters.institution_id} deleted`, 500);
    }
    return {
        status: 204,
    };
};
exports.deleteInstitutionsByInstitutionId = deleteInstitutionsByInstitutionId;
//# sourceMappingURL=institutions.js.map